@@ -14,6 +14,7 @@ import requests
 import copy
 import json
 from urllib.parse import urlparse
+from urllib.request import urlopen
 from OpenSSL.crypto import load_certificate, FILETYPE_PEM
 import yaml
 
@@ -24,28 +25,30 @@ class CommandUtils(object):
         self.hostRpmIsNotUsable = -1
 
     def run(self, cmd, update_env=False):
-        self.logger.debug(cmd)
+        self.logger.info(f"running {cmd}")
         use_shell = not isinstance(cmd, list)
         process = subprocess.Popen(
-            cmd, shell=use_shell, stdout=subprocess.PIPE, stderr=subprocess.PIPE
+            cmd, shell=use_shell, text=True,
+            stdout=subprocess.PIPE, stderr=subprocess.PIPE
         )
         out, err = process.communicate()
-        retval = process.returncode
-        if out != b"":
-            self.logger.info(out.decode())
+        if out != "":
+            self.logger.info(out)
             if update_env:
                 os.environ.clear()
                 os.environ.update(
                     dict(
                         line.partition("=")[::2]
-                        for line in out.decode("utf8").split("\0")
+                        for line in out.split("\0")
                         if line
                     )
                 )
+        process.wait()
+        retval = process.returncode
         if retval != 0:
             self.logger.info("Command failed: {}".format(cmd))
             self.logger.info("Error code: {}".format(retval))
-            self.logger.error(err.decode())
+            self.logger.error(err)
         return retval
 
     def run_in_chroot(self, chroot_path, cmd, update_env=False):
@@ -103,6 +106,28 @@ class CommandUtils(object):
         except FileNotFoundError:
             return False
 
+    # check if url is a URL (note: a file path is not a URL)
+    @staticmethod
+    def is_url(url):
+        try:
+            result = urlparse(url)
+            return all([result.scheme, result.netloc])
+        except:
+            return False
+
+    # read json from a file or URL
+    @staticmethod
+    def load_json(url):
+        if CommandUtils.is_url(url):
+            with urlopen(url) as f:
+                data = json.load(f)
+        else:
+            if url.startswith("file://"):
+                url = url[7:]
+            with open(url, "rt") as f:
+                data = json.load(f)
+        return data
+
     @staticmethod
     def wget(url, out, enforce_https=True, ask_fn=None, fingerprint=None):
         # Check URL
@@ -182,12 +207,16 @@ class CommandUtils(object):
 
         assert type(key) is str, f"param name must be a string"
 
-        if "=" in key:
-            key, default = [t.strip() for t in key.split("=")]
-            default = yaml.safe_load(default)
-        value = params.get(key, default)
+        if '=' in key:
+            key, default = [t.strip() for t in key.split('=', maxsplit=1)]
 
-        assert value is not None, f"no param set for '{key}', and there is no default"
+            if key in params:
+                value = params[key]
+            else:
+                value = yaml.safe_load(default)
+        else:
+            assert key in params, f"no param set for '{key}', and there is no default"
+            value = params[key]
 
         return value
 
@@ -246,20 +275,6 @@ class CommandUtils(object):
             json.dump(packages_list, json_file, indent=4)
         return file_path
 
-    @staticmethod
-    def merge_json_files(merged_file, file1, file2):
-        data1 = CommandUtils.jsonread(file1)
-        data2 = CommandUtils.jsonread(file2)
-        merged_data = {}
-        keys = ["packages", "packages_aarch64", "packages_x86_64"]
-
-        for key in keys:
-            merged_list = list(set(data1.get(key, []) + data2.get(key, [])))
-            if merged_list:
-                merged_data[key] = merged_list
-
-        return CommandUtils.write_pkg_list_file(merged_file, merged_data)
-
     def replace_in_file(self, file_path, pattern, replacement):
         try:
             with open(file_path, "r", encoding="utf-8") as file:
@@ -304,3 +319,30 @@ class CommandUtils(object):
             pass
         except Exception as e:
             raise Exception(f"Error removing {file_path}: {e}")
+
+    def acquire_file_map(self, map, dest_dir):
+        """
+        map is a dictionary that maps source files to destinations
+        the sources can be URLs or files.
+        The destinations are paths under dest_dir. Any directories in the
+        paths will be created if needed.
+        If the basename of the destination is just a directory, the basename
+        of the source will be used.
+        """
+        for src, dest in map.items():
+            if dest.startswith("/"):
+                dest = dest[1:]
+            if os.path.basename(dest) == "":
+                dest = os.path.join(os.path.dirname(dest), os.path.basename(src))
+            dest = os.path.join(dest_dir, dest)
+            os.makedirs(os.path.dirname(dest), exist_ok=True)
+
+            if src.startswith("file://"):
+                src = src[7:]
+            if CommandUtils.is_url(src):
+                self.logger.info(f"downloading {src} to {dest}")
+                ret, _ = CommandUtils.wget(src, dest)
+                assert ret, f"downloading {src} failed"
+            else:
+                self.logger.info(f"copying {src} to {dest}")
+                shutil.copyfile(src, dest)
